﻿using UnityEngine;
using System.Collections.Generic;

using BOOL = System.Boolean;


namespace Ardunity
{
	[AddComponentMenu("ARDUnity/Controller/Basic/DigitalOutput")]
    [HelpURL("https://sites.google.com/site/ardunitydoc/references/controller/digitaloutput")]
	public class DigitalOutput : ArdunityController, IWireOutput<bool>, IWireOutput<float>
	{
		public int pin;	
		
        [SerializeField]
		private BOOL _value = false;
		
		protected override void Awake()
		{
			base.Awake();
			
			enableUpdate = false; // only output.
		}
		
		protected override void OnPush()
		{
			Push(_value);
		}
		
		protected override void OnPop()
		{ 			 
		}
		
		public override string GetCodeDeclaration()
		{
			return string.Format("{0} {1}({2:d}, {3:d});", this.GetType().Name, GetCodeVariable(), id, pin);
		}
		
		public override string GetCodeVariable()
		{
			return string.Format("dOutput{0:d}", id);
		}
		
		public bool Value
		{
			get
			{
				return (bool)_value;
			}
			set
			{
				if(_value != value)
				{
					_value = value;
					SetDirty();
				}
			}
		}
		
        #region Wire Editor
		bool IWireOutput<bool>.output
        {
			get
			{
				return Value;
			}
			set
			{
				Value = value;
			}
		}
		
		float IWireOutput<float>.output
        {
			get
			{
				if(Value)
					return 1f;
				else
					return 0f;
			}
			set
			{
				if(Mathf.Abs(value) < 0.5f)
					Value = false;
				else
					Value = true;
			}
		}
       
		protected override void AddNode(List<Node> nodes)
        {
			base.AddNode(nodes);
            
            nodes.Add(new Node("pin", "", null, NodeType.None, "Arduino Digital Pin"));
            nodes.Add(new Node("digitalValue", "Value(digital)", typeof(IWireOutput<bool>), NodeType.WireTo, "Output<bool>"));
			nodes.Add(new Node("analogValue", "Value(analog)", typeof(IWireOutput<float>), NodeType.WireTo, "Output<float>"));
        }
        
        protected override void UpdateNode(Node node)
        {
            if(node.name.Equals("pin"))
            {
                node.text = string.Format("Pin: {0:d}", pin);
                return;
            }
            
            base.UpdateNode(node);
        }
        #endregion
	}
}
